import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.InputMismatchException;

import model.*;
import utils.*;

public class Runner implements Runnable {

	private Socket socket;
	private InputStreamReader in;
	private OutputStreamWriter out;
	private Strategy strategy;
	private String token;
	
	public Runner(Socket socket, Strategy strategy, String token) throws IOException
	{
		this.socket = socket;
		this.in = new InputStreamReader(socket.getInputStream());
		this.out = new OutputStreamWriter(socket.getOutputStream());
		this.token = token;
		this.strategy = strategy;
	}
	
	public void disconnect()
	{
		try
		{
			socket.close();
		} catch (IOException e)
		{
			e.printStackTrace();
		}
		
	}

	@Override
	public void run()
	{
		try
		{
			out.write(token);
			out.flush();
			
			int c;
			while( (c = in.read()) != -1)
			{
				switch(c)
				{
				case 'D':
					disconnect();
					break;
				case 'A':{
					Player player = StringUtils.readPlayer(in);
					World world = StringUtils.readWorld(in);
					
					Action a = strategy.getAction(player, world);
					
					StringUtils.writeToken(out, a);
					out.flush();
					
					break;
				}
				case 'I':{
					Rules rules = StringUtils.readRules(in);
					strategy.init(rules);
					break;
				}
				default:
					throw new InputMismatchException("Unknown action " + c);
				}
			}
		}
		catch(IOException | InputMismatchException e)
		{
			e.printStackTrace();
			disconnect();
		}
		
	}
	
	public static void main(String[] args) throws UnknownHostException, IOException
	{
        String host = args.length < 1 ? "127.0.0.1" : args[0];
        int port = args.length < 2 ? 7828 : Integer.parseInt(args[1]);
        String token = args.length < 3 ? "0000000000000000000000000000000000000000000000000000000000000000" : args[2];
                
        Socket socket = new Socket(host, port);
        Runner runner = new Runner(socket, new MyStrategy(), token);
        runner.run();
		
	}

}
