package utils;

import model.*;

public class WorldGeometry {
	public static final double EPS = 1e-12;
	public static boolean checkCollision(WorldObject a, WorldObject b)
	{
		if(a.isCircle() && b.isCircle())
			return checkCircleCircleCollision(a.getX(), a.getY(), a.size/2.0, b.getX(), b.getY(), b.size/2.0);
		if(a.isCircle() && b.isSquare())
			return checkCircleRectangleCollision(a.getX(), a.getY(), a.size/2.0, b.getX() - b.size/2.0, b.getY() - b.size/2.0, b.getX() + b.size/2.0, b.getY() + b.size/2.0);
		if(a.isSquare() && b.isCircle())
			return checkCircleRectangleCollision(b.getX(), b.getY(), b.size/2.0, a.getX() - a.size/2.0, a.getY() - a.size/2.0, a.getX() + a.size/2.0, a.getY() + a.size/2.0);
		if(a.isSquare() && b.isSquare())
		{
			return checkRectangleRectangleCollision(
					a.getX() - a.size/2.0, a.getY() - a.size/2.0, a.getX() + a.size/2.0, a.getY() + a.size/2.0,
					b.getX() - b.size/2.0, b.getY() - b.size/2.0, b.getX() + b.size/2.0, b.getY() + b.size/2.0);
		}
		return false;
	}
	
	public static boolean checkRectangleRectangleCollision(double xLo1, double yLo1, double xHi1, double yHi1,
					double xLo2, double yLo2, double xHi2, double yHi2)
	{
		double xLo = Math.max(xLo1, xLo2);
		double xHi = Math.min(xHi1, xHi2);
		double yLo = Math.max(yLo1, yLo2);
		double yHi = Math.min(yHi1, yHi2);
		
		return xLo+EPS < xHi && yLo+EPS < yHi;
	}
	
	public static boolean checkCircleCircleCollision(double x1, double y1, double r1, double x2, double y2, double r2)
	{
		return getPointToCircleDistance(x1, y1, x2, y2, r1+r2)<=EPS;
	}
	
	public static boolean checkCircleRectangleCollision(double xC, double yC, double rC, double xLo, double yLo, double xHi, double yHi)
	{
		return Math.min(
				Math.min(
						getPointToRectangleDistance(xC, yC, xLo, yLo-rC, xHi, yHi+rC), 
						getPointToRectangleDistance(xC, yC, xLo-rC, yLo, xHi+rC, yHi)),
				Math.min(
					Math.min(
						getPointToCircleDistance(xC, yC, xLo, yLo, rC),
						getPointToCircleDistance(xC, yC, xLo, yHi, rC)),
					Math.min(
						getPointToCircleDistance(xC, yC, xHi, yLo, rC),
						getPointToCircleDistance(xC, yC, xHi, yHi, rC))
					)
				)<=EPS;
	}
	
	public static double getPointToRectangleDistance(double xP, double yP, double xLo, double yLo, double xHi, double yHi)
	{
		if(xLo<=xP && xP<=xHi && yLo<=yP && yP<=yHi)
			return 0.0;
		double xM = Math.min(Math.max(xP, xLo), xHi);
		double yM = Math.min(Math.max(yP, yLo), yHi);
		return Math.hypot(xP-xM, yP-yM);
	}
	
	public static double getPointToCircleDistance(double xP, double yP, double xC, double yC, double rC)
	{
		return Math.max(0.0, (Math.hypot(xP-xC, yP-yC) - rC));
	}
}
